@@ -1,5 +1,4 @@
 # frozen_string_literal: true
-require 'ipaddr'
 
 module Excon
   class Connection
@@ -62,6 +61,7 @@ module Excon
     # @option params [Class] :instrumentor Responds to #instrument as in ActiveSupport::Notifications
     # @option params [String] :instrumentor_name Name prefix for #instrument events.  Defaults to 'excon'
     def initialize(params = {})
+      @pid = Process.pid
       @data = Excon.defaults.dup
       # merge does not deep-dup, so make sure headers is not the original
       @data[:headers] = @data[:headers].dup
@@ -89,7 +89,9 @@ module Excon
       end
 
       if @data[:scheme] == UNIX
-        if @data[:host]
+        # 'uri' >= v0.12.0 returns an empty string instead of nil for no host.
+        # So treat the parameter as present if and only if it is both non-nill and non-empty.
+        if @data[:host] && !@data[:host].empty?
           raise ArgumentError, "The `:host` parameter should not be set for `unix://` connections.\n" +
                                "When supplying a `unix://` URI, it should start with `unix:/` or `unix:///`."
         elsif !@data[:socket]
@@ -227,6 +229,13 @@ module Excon
     def request(params={}, &block)
       # @data has defaults, merge in new params to override
       datum = @data.merge(params)
+
+      # Set the deadline for the current request in order to determine when we have run out of time.
+      # Only set when a request timeout has been defined.
+      if datum[:timeout]
+        datum[:deadline] = Process.clock_gettime(Process::CLOCK_MONOTONIC) + datum[:timeout]
+      end
+
       datum[:headers] = @data[:headers].merge(datum[:headers] || {})
 
       validate_params(:request, params, datum[:middlewares])
@@ -241,16 +250,22 @@ module Excon
         datum[:headers]['Authorization'] ||= 'Basic ' + ["#{user}:#{pass}"].pack('m').delete(Excon::CR_NL)
       end
 
+      host_key = datum[:headers].keys.detect {|k| k.casecmp('Host') == 0 } || 'Host'
       if datum[:scheme] == UNIX
-        datum[:headers]['Host']   ||= ''
+        datum[:headers][host_key] ||= ''
       else
-        datum[:headers]['Host']   ||= datum[:host] + port_string(datum)
+        datum[:headers][host_key] ||= datum[:host] + port_string(datum)
       end
 
       # RFC 7230, section 5.4, states that the Host header SHOULD be the first one # to be present.
       # Some web servers will reject the request if it comes too late, so let's hoist it to the top.
-      if (host = datum[:headers].delete('Host'))
-        datum[:headers] = { 'Host' => host }.merge(datum[:headers])
+      if (host = datum[:headers].delete(host_key))
+        datum[:headers] = { host_key => host }.merge(datum[:headers])
+      end
+
+      # default to GET if no method specified
+      unless datum[:method]
+        datum[:method] = :get
       end
 
       # if path is empty or doesn't start with '/', insert one
@@ -473,6 +488,11 @@ module Excon
       @_excon_sockets ||= {}
       @_excon_sockets.compare_by_identity
 
+      if @pid != Process.pid
+        @_excon_sockets.clear # GC will take care of closing sockets
+        @pid = Process.pid
+      end
+
       if @data[:thread_safe_sockets]
         # In a multi-threaded world, if the same connection is used by multiple
         # threads at the same time to connect to the same destination, they may
@@ -572,6 +592,8 @@ module Excon
           raise ArgumentError, "The `:ssl_proxy_headers` parameter should only be used with HTTPS requests."
         end
         if @data[:proxy][:scheme] == UNIX
+          # URI.parse might return empty string for security reasons.
+          @data[:proxy][:host] = nil if @data[:proxy][:host] == ""
           if @data[:proxy][:host]
             raise ArgumentError, "The `:host` parameter should not be set for `unix://` proxies.\n" +
                                  "When supplying a `unix://` URI, it should start with `unix:/` or `unix:///`."
