@@ -54,11 +54,13 @@ module Excon
           ssl_context.cert_store = cert_store
         end
 
-        # no defaults, fallback to bundled
-        unless ca_file || ca_path || cert_store
+        if cert_store.nil?
           ssl_context.cert_store = OpenSSL::X509::Store.new
           ssl_context.cert_store.set_default_paths
+        end
 
+        # no defaults, fallback to bundled
+        unless ca_file || ca_path || cert_store
           # workaround issue #257 (JRUBY-6970)
           ca_file = DEFAULT_CA_FILE
           ca_file = ca_file.gsub(/^jar:/, '') if ca_file =~ /^jar:file:\//
@@ -88,6 +90,14 @@ module Excon
         else
           ssl_context.key = OpenSSL::PKey::RSA.new(client_key_data, client_key_pass)
         end
+        if client_chain_data && OpenSSL::X509::Certificate.respond_to?(:load)
+          ssl_context.extra_chain_cert = OpenSSL::X509::Certificate.load(client_chain_data)
+        elsif client_chain_data
+          certs = client_chain_data.scan(/-----BEGIN CERTIFICATE-----(?:.|\n)+?-----END CERTIFICATE-----/)
+          ssl_context.extra_chain_cert = certs.map do |cert|
+            OpenSSL::X509::Certificate.new(cert)
+          end
+        end
       elsif @data.key?(:certificate) && @data.key?(:private_key)
         ssl_context.cert = OpenSSL::X509::Certificate.new(@data[:certificate])
         if OpenSSL::PKey.respond_to? :read
@@ -131,7 +141,7 @@ module Excon
 
       # Server Name Indication (SNI) RFC 3546
       if @socket.respond_to?(:hostname=)
-        @socket.hostname = @data[:host]
+        @socket.hostname = @data[:ssl_verify_peer_host] || @data[:host]
       end
 
       begin
@@ -169,6 +179,14 @@ module Excon
                             end
     end
 
+    def client_chain_data
+      @client_chain_data ||= if (ccd = @data[:client_chain_data])
+                               ccd
+                             elsif (path = @data[:client_chain])
+                               File.read path
+                             end
+    end
+
     def connect
       # backwards compatability for things lacking nonblock
       @nonblock = HAVE_NONBLOCK && @nonblock
