@@ -4,7 +4,7 @@ module Excon
       module Server
         module Puma
           def start(app_str = app, bind_uri = bind)
-            open_process('puma', '-b', bind_uri.to_s, app_str)
+            open_process(RbConfig.ruby, '-S', 'puma', '-b', bind_uri.to_s, app_str)
             process_stderr = ""
             line = ''
             until line =~ /Use Ctrl-C to stop/
