@@ -1,8 +1,8 @@
-require 'rubygems' if RUBY_VERSION < '1.9'
 require 'bundler/setup'
 require 'excon'
 require 'delorean'
 require 'open4'
+require 'webrick'
 
 require './spec/helpers/warning_helpers.rb'
 
@@ -24,6 +24,33 @@ def basic_tests(url = 'http://127.0.0.1:9292', options = {})
 
     tests("nonblock => #{nonblock}, persistent => #{persistent}") do
 
+      tests('method and path defaults') do
+        tests('request().body').returns('GET /') do
+          response = connection.request(:path => '/')
+          response.body
+        end
+
+        tests("request(:headers => {'host' => '127.0.0.1'})").returns('GET /') do
+          response = connection.request(:headers => {'host' => '127.0.0.1'})
+          response.body
+        end
+
+        tests('request(:method => :get).body').returns('GET /') do
+          response = connection.request(:method => :get)
+          response.body
+        end
+
+        tests("request(:path => '/').body").returns('GET /') do
+          response = connection.request(:path => '/')
+          response.body
+        end
+
+        tests("request(:method => :get, :path => '/').body").returns('GET /') do
+          response = connection.request(:method => :get, :path => '/')
+          response.body
+        end
+      end
+
       tests('GET /content-length/100') do
         response = nil
 
@@ -104,15 +131,14 @@ def basic_tests(url = 'http://127.0.0.1:9292', options = {})
       end
 
       tests('POST /echo') do
-
         tests('with file').returns('x' * 100 + "\n") do
           file_path = File.join(File.dirname(__FILE__), "data", "xs")
-          response = connection.request(:method => :post, :path => '/echo', :body => File.open(file_path))
+          response = connection.request(:method => :post, :path => '/echo', :headers => { 'Content-Type' => 'text/plain' }, :body => File.open(file_path))
           response.body
         end
 
         tests('without request_block').returns('x' * 100) do
-          response = connection.request(:method => :post, :path => '/echo', :body => 'x' * 100)
+          response = connection.request(:method => :post, :path => '/echo', :headers => { 'Content-Type' => 'text/plain' }, :body => 'x' * 100)
           response.body
         end
 
@@ -121,24 +147,22 @@ def basic_tests(url = 'http://127.0.0.1:9292', options = {})
           request_block = lambda do
             data.shift.to_s
           end
-          response = connection.request(:method => :post, :path => '/echo', :request_block => request_block)
+          response = connection.request(:method => :post, :path => '/echo', :headers => { 'Content-Type' => 'text/plain' }, :request_block => request_block)
           response.body
         end
 
         tests('with multi-byte strings') do
           body = "\xC3\xBC" * 100
           headers = { 'Custom' => body.dup }
-          if RUBY_VERSION >= '1.9'
-            body.force_encoding('BINARY')
-            headers['Custom'].force_encoding('UTF-8')
-          end
+          body.force_encoding('BINARY')
+          headers['Custom'].force_encoding('UTF-8')
+          headers['Content-Type'] = 'text/plain'
 
           returns(body, 'properly concatenates request+headers and body') do
             response = connection.request(:method => :post, :path => '/echo', :headers => headers, :body => body)
             response.body
           end
         end
-
       end
 
       tests('PUT /echo') do
@@ -166,10 +190,8 @@ def basic_tests(url = 'http://127.0.0.1:9292', options = {})
         tests('with multi-byte strings') do
           body = "\xC3\xBC" * 100
           headers = { 'Custom' => body.dup }
-          if RUBY_VERSION >= '1.9'
-            body.force_encoding('BINARY')
-            headers['Custom'].force_encoding('UTF-8')
-          end
+          body.force_encoding('BINARY')
+          headers['Custom'].force_encoding('UTF-8')
 
           returns(body, 'properly concatenates request+headers and body') do
             response = connection.request(:method => :put, :path => '/echo', :headers => headers, :body => body)
@@ -191,6 +213,76 @@ def basic_tests(url = 'http://127.0.0.1:9292', options = {})
 end
 
 
+# expected values: the response, in pieces, and a timeout after each piece
+STREAMING_PIECES = %w{Hello streamy world}
+STREAMING_TIMEOUT = 0.1
+
+def streaming_tests(protocol)
+  conn = nil
+  test do
+    conn = Excon.new("#{protocol}://127.0.0.1:9292/", :ssl_verify_peer => false)
+    true
+  end
+
+  # expect the full response as a string
+  # and expect it to take a (timeout * pieces) seconds
+  tests('simple blocking request on streaming endpoint').returns([STREAMING_PIECES.join(''),'response time ok']) do
+    start = Time.now
+    ret = conn.request(:method => :get, :path => '/streamed/simple').body
+
+    if Time.now - start <= STREAMING_TIMEOUT*3
+      [ret, 'streaming response came too quickly']
+    else
+      [ret, 'response time ok']
+    end
+  end
+
+  # expect the full response as a string and expect it to
+  # take a (timeout * pieces) seconds (with fixed Content-Length header)
+  tests('simple blocking request on streaming endpoint with fixed length').returns([STREAMING_PIECES.join(''),'response time ok']) do
+    start = Time.now
+    ret = conn.request(:method => :get, :path => '/streamed/fixed_length').body
+
+    if Time.now - start <= STREAMING_TIMEOUT*3
+      [ret, 'streaming response came too quickly']
+    else
+      [ret, 'response time ok']
+    end
+  end
+
+  # expect each response piece to arrive to the body right away
+  # and wait for timeout until next one arrives
+  def timed_streaming_test(conn, path, timeout)
+    ret = []
+    timing = 'response times ok'
+    start = Time.now
+    conn.request(:method => :get, :path => path, :response_block => lambda do |c,r,t|
+      # add the response
+      ret.push(c)
+      # check if the timing is ok
+      # each response arrives after timeout and before timeout + 1
+      cur_time = Time.now - start
+      if cur_time < ret.length * timeout or cur_time > (ret.length+1) * timeout
+        timing = 'response time not ok!'
+      end
+    end)
+    # validate the final timing
+    if Time.now - start <= timeout*3
+      timing = 'final timing was not ok!'
+    end
+    [ret, timing]
+  end
+
+  tests('simple request with response_block on streaming endpoint').returns([STREAMING_PIECES,'response times ok']) do
+    timed_streaming_test(conn, '/streamed/simple', STREAMING_TIMEOUT)
+  end
+
+  tests('simple request with response_block on streaming endpoint with fixed length').returns([STREAMING_PIECES,'response times ok']) do
+    timed_streaming_test(conn, '/streamed/fixed_length', STREAMING_TIMEOUT)
+  end
+end
+
+
 PROXY_ENV_VARIABLES = %w{http_proxy https_proxy no_proxy} # All lower-case
 
 def env_init(env={})
@@ -224,7 +316,6 @@ end
 
 def launch_process(*args)
   unless RUBY_PLATFORM == 'java'
-    GC.disable if RUBY_VERSION < '1.9'
     pid, w, r, e = Open4.popen4(*args)
   else
     pid, w, r, e = IO.popen4(*args)
@@ -235,7 +326,6 @@ end
 def cleanup_process(pid)
   Process.kill(9, pid)
   unless RUBY_PLATFORM == 'java'
-    GC.enable if RUBY_VERSION < '1.9'
     Process.wait(pid)
   end
 end
@@ -245,7 +335,7 @@ def rackup_path(*parts)
 end
 
 def with_rackup(name, host="127.0.0.1")
-  pid, w, r, e = launch_process("rackup", "-s", "webrick", "--host", host, rackup_path(name))
+  pid, w, r, e = launch_process(RbConfig.ruby, "-S", "rackup", "-s", "webrick", "--host", host, rackup_path(name))
   process_stderr = ""
   until (line = e.gets) =~ /HTTPServer#start:/
     raise process_stderr if line.nil?
@@ -274,7 +364,7 @@ end
 def with_unicorn(name, listen='127.0.0.1:9292')
   unless RUBY_PLATFORM == 'java'
     unix_socket = listen.sub('unix://', '') if listen.start_with? 'unix://'
-    pid, w, r, e = launch_process("unicorn", "--no-default-middleware","-l", listen, rackup_path(name))
+    pid, w, r, e = launch_process(RbConfig.ruby, "-S", "unicorn", "--no-default-middleware","-l", listen, rackup_path(name))
     process_stderr = ""
     until (line = e.gets) =~ /worker=0 ready/
       raise process_stderr if line.nil?
@@ -297,7 +387,7 @@ def server_path(*parts)
 end
 
 def with_server(name)
-  pid, w, r, e = launch_process("ruby", server_path("#{name}.rb"))
+  pid, w, r, e = launch_process(RbConfig.ruby, server_path("#{name}.rb"))
   process_stderr = ""
   until (line = e.gets) =~ /ready/
     raise process_stderr if line.nil?
@@ -307,3 +397,51 @@ def with_server(name)
 ensure
   cleanup_process(pid)
 end
+
+# A tiny fake SSL streaming server
+def with_ssl_streaming(port, pieces, delay)
+  key_file = File.join(File.dirname(__FILE__), 'data', '127.0.0.1.cert.key')
+  cert_file = File.join(File.dirname(__FILE__), 'data', '127.0.0.1.cert.crt')
+
+  ctx = OpenSSL::SSL::SSLContext.new
+  ctx.key = OpenSSL::PKey::RSA.new(File.read(key_file))
+  ctx.cert = OpenSSL::X509::Certificate.new(File.read(cert_file))
+
+  tcp = TCPServer.new(port)
+  ssl = OpenSSL::SSL::SSLServer.new(tcp, ctx)
+
+  Thread.new do
+    loop do
+      begin
+        conn = ssl.accept
+      rescue IOError => e
+        # we're closing the socket from another thread, which makes `accept` complain
+        break if /stream closed/ =~ e.to_s
+        raise
+      end
+
+      Thread.new do
+        begin
+          req = WEBrick::HTTPRequest.new(WEBrick::Config::HTTP)
+          req.parse(conn)
+
+          conn << "HTTP/1.1 200 OK\r\n\r\n"
+          if req.path == "streamed/fixed_length"
+            conn << "Content-Length: #{pieces.join.length}\r\n"
+          end
+          conn.flush
+
+          pieces.each do |piece|
+            sleep(delay)
+            conn.write(piece)
+            conn.flush
+          end
+        ensure
+          conn.close
+        end
+      end
+    end
+  end
+  yield
+  ssl.close
+end
